﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("Exhaustive MultiComponentVector Triple Move Generator", "Generates all possible multi component moves from a given multiComponentVector.")]
  [StorableClass]
  public class ExhaustiveMCVTripleMoveGenerator : MultiComponentVectorMoveGenerator, IExhaustiveMoveGenerator {
    [StorableConstructor]
    protected ExhaustiveMCVTripleMoveGenerator(bool deserializing) : base(deserializing) { }
    protected ExhaustiveMCVTripleMoveGenerator(ExhaustiveMCVTripleMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public ExhaustiveMCVTripleMoveGenerator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExhaustiveMCVTripleMoveGenerator(this, cloner);
    }

    public static IEnumerable<MultiComponentVectorMove> GenerateMCVTripleMoves(MultiComponentVectorEncoding mcv) {
      int nrOfBins = 0;
      foreach (var pi in mcv.PackingInformations) {
        if (pi.AssignedBin > nrOfBins)
          nrOfBins = pi.AssignedBin;
      }
      nrOfBins++;
      Dictionary<int, List<int>> indexesPerBin = new Dictionary<int, List<int>>();
      for (int i = 0; i < mcv.PackingInformations.Count; i++) {
        int currentBin = mcv.PackingInformations[i].AssignedBin;
        if (!indexesPerBin.ContainsKey(currentBin))
          indexesPerBin[currentBin] = new List<int>();
        indexesPerBin[currentBin].Add(i);
      }
                                                        
      for (int index = 0; index < mcv.PackingInformations.Count; index++) { 
        for (int group = 0; group < nrOfBins; group++) {
          foreach (int targetIndex in indexesPerBin[group]) {
            yield return new MCVTripleMove(index, false, group, targetIndex, mcv);
            yield return new MCVTripleMove(index, true, group, targetIndex, mcv);
          }
        }
      }
    }

    public static MultiComponentVectorMove[] Apply(MultiComponentVectorEncoding multiComponentVector) {
      var generatedMoves = new List<MultiComponentVectorMove>(GenerateMCVTripleMoves(multiComponentVector));
      int totalMoves = generatedMoves.Count;
      MultiComponentVectorMove[] moves = new MultiComponentVectorMove[totalMoves];
      int count = 0;
      foreach (MultiComponentVectorMove move in generatedMoves) {
        moves[count++] = move;
      }
      return moves;
    }  

    protected override MultiComponentVectorMove[] GenerateMoves(MultiComponentVectorEncoding multiComponentVector) {
      return Apply(multiComponentVector);
    }
  }
}
