﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("MCVTripleMove", "A move on a multi component vector that is specified by a rotation-value, two indexes and a groupNr.")]
  [StorableClass]
  public class MCVTripleMove : MultiComponentVectorMove {
    [Storable]
    public int Index { get; protected set; }
    [Storable]
    public bool Rotation { get; protected set; }
    [Storable]
    public int Group { get; protected set; }
    [Storable]
    public int TargetIndex { get; protected set; }

    [StorableConstructor]
    protected MCVTripleMove(bool deserializing) : base(deserializing) { }
    protected MCVTripleMove(MCVTripleMove original, Cloner cloner)
      : base(original, cloner) {
      this.Index = original.Index;
      this.Rotation = original.Rotation;
      this.Group = original.Group;
      this.TargetIndex = original.TargetIndex;
    }
    public MCVTripleMove(int index, bool rotation, int group, int targetIndex, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        Index = index;
        Rotation = rotation;
        Group = group;
        TargetIndex = targetIndex;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MCVTripleMove(this, cloner);
    }

    public override MultiComponentVectorEncoding GetVectorAfterMove() {
      var result = MultiComponentVector;
      if (Rotation)
        result = SingleItemRotationMove.GetVectorAfterMove(MultiComponentVector, Index);
      if (!MultiComponentVector.PackingInformations[Index].AssignedBin.Equals(Group))
        result = SingleGroupingMove.GetVectorAfterMove(result, Index, Group);
      if (!Index.Equals(TargetIndex))
        result = ChangePositionMove.GetVectorAfterMove(result, Index, TargetIndex);
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(MCVTripleMoveAttribute);
    }

    public override string ToString() {
      return "TM(i=" + Index + ",r=" + Rotation + ",g=" + Group + ",ti=" + TargetIndex + ")";
    }
  }
}