﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SwapPositionMove", "A move on a multi component vector that is specified by two indexes. The packing informations at the speciefied indexes are beeing swapped.")]
  [StorableClass]
  public class SwapPositionMove : MultiComponentVectorMove {
    [Storable]
    public int Index1 { get; protected set; }
    [Storable]
    public int Index2 { get; protected set; }

    [StorableConstructor]
    protected SwapPositionMove(bool deserializing) : base(deserializing) { }
    protected SwapPositionMove(SwapPositionMove original, Cloner cloner)
      : base(original, cloner) {
      this.Index1 = original.Index1;
      this.Index2 = original.Index2;
    }
    public SwapPositionMove(int index1, int index2, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        Index1 = index1;
        Index2 = index2;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SwapPositionMove(this, cloner);
    }

    public override MultiComponentVectorEncoding GetVectorAfterMove() {
      return GetVectorAfterMove(MultiComponentVector, Index1, Index2);
    }
    public static MultiComponentVectorEncoding GetVectorAfterMove(MultiComponentVectorEncoding multiComponentVector, int index1, int index2) {
      var result = multiComponentVector.Clone(new Cloner()) as MultiComponentVectorEncoding;
      var aux = result.PackingInformations[index1];
      result.PackingInformations[index1] = result.PackingInformations[index2];
      result.PackingInformations[index2] = aux;
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(SwapPositionMoveAttribute);
    }

    public override string ToString() {
      return "SM(i1=" + Index1 + ",i2=" + Index2 + ")";
    }
  }
}