﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("Uniform Multi Component Vector Manipulator", "An operator which manipulates a PackingSequence representation.")]
  [StorableClass]
  public class ThreeWayMultiComponentVectorManipulator :  MultiComponentVectorManipulator {
    [StorableConstructor]
    protected ThreeWayMultiComponentVectorManipulator(bool deserializing) : base(deserializing) { }
    protected ThreeWayMultiComponentVectorManipulator(ThreeWayMultiComponentVectorManipulator original, Cloner cloner) : base(original, cloner) { }
    public ThreeWayMultiComponentVectorManipulator()
      : base() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ThreeWayMultiComponentVectorManipulator(this, cloner);
    }

    protected override void Manipulate(IRandom random, MultiComponentVectorEncoding individual) {
      int manipulatorDecision = random.Next(0, 3);

      //Randomly rotate one item
      if (manipulatorDecision == 0) {
        bool r = individual.PackingInformations[random.Next(individual.PackingInformations.Count - 1)].Rotated;
        individual.PackingInformations[random.Next(individual.PackingInformations.Count - 1)].Rotated = !r;
      } 
      //Randomly change the bin-assignment for one item
      else if (manipulatorDecision == 1) {
        individual.PackingInformations[random.Next(individual.PackingInformations.Count - 1)].AssignedBin = 0;
      }
      //Swap the sequence of two items assigned to the same bin
      else if (manipulatorDecision == 2) {
        int length = individual.PackingInformations.Count;

        int counter = 0;
        int swappedIndex1 = random.Next(length); 
        int swappedIndex2 = random.Next(length);
        while ((swappedIndex1 == swappedIndex2 || individual.PackingInformations[swappedIndex1].AssignedBin != individual.PackingInformations[swappedIndex2].AssignedBin) && counter < 10)
        {
          swappedIndex1 = random.Next(length); 
          swappedIndex2 = random.Next(length);
          counter++;
        }

        if (swappedIndex1 != swappedIndex2 && individual.PackingInformations[swappedIndex1].AssignedBin == individual.PackingInformations[swappedIndex2].AssignedBin) {
          int temp = individual.PackingInformations[swappedIndex1].ItemIndex;
          individual.PackingInformations[swappedIndex1].ItemIndex = individual.PackingInformations[swappedIndex2].ItemIndex;
          individual.PackingInformations[swappedIndex2].ItemIndex = temp;
        }

      }

    }

  }
}
