﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.Potvin {
  [Item("Potvin Encoding Crossover", "An operator which crosses two Potvin representations. According to RBX by Potvin.")]
  [StorableClass]
  public class PotvinCrossover : PackingSolutionCrossover, IPotvinOperator {

    [StorableConstructor]
    protected PotvinCrossover(bool deserializing) : base(deserializing) { }
    protected PotvinCrossover(PotvinCrossover original, Cloner cloner) : base(original, cloner) { }
    public PotvinCrossover()
      : base() {
        ParentsParameter.ActualName = "Potvin";
        ChildParameter.ActualName = "Potvin";
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PotvinCrossover(this, cloner);
    }

    public PotvinEncoding Cross(IRandom random, PotvinEncoding parent1, PotvinEncoding parent2) {
      PotvinEncoding child = parent2.Clone() as PotvinEncoding;

      if (parent1.NrOfBins > 0 && child.NrOfBins > 0) {
        int tourParent1 = random.Next(parent1.NrOfBins);
        List<int> replacing = parent1.GetItemsInBin(tourParent1);

        int tourParent2 = random.Next(child.NrOfBins);
        List<int> replaced = child.GetItemsInBin(tourParent2);

        child.Tours.Remove(replaced);
        child.Tours.Insert(tourParent2, replacing);

        Permutation vehicleAssignment = child.VehicleAssignment;

        int vehicle = vehicleAssignment[tourParent2];
        int vehicle2 = parent1.VehicleAssignment[tourParent1];
        vehicleAssignment[tourParent2] = vehicle2;

        for (int i = 0; i < vehicleAssignment.Length; i++) {
          if (vehicleAssignment[i] == vehicle2 && i != tourParent2) {
            vehicleAssignment[i] = vehicle;
            break;
          }
        }

        foreach (int city in replaced.Stops)
          if (FindRoute(child, city) == null && !child.Unrouted.Contains(city))
            child.Unrouted.Add(city);

        if (Repair(random, child, replacing, problemInstance, allowInfeasible) || allowInfeasible)
          return child;
        else {
          if (random.NextDouble() < 0.5)
            return parent1.Clone() as PotvinEncoding;
          else
            return parent2.Clone() as PotvinEncoding;
        }
      } else {
        return child;
      }
    }

    public override IOperation Apply() {
      var parents = ParentsParameter.ActualValue;

      ChildParameter.ActualValue =
        Cross(RandomParameter.ActualValue, parents[0] as PotvinEncoding, parents[1] as PotvinEncoding);

      return base.Apply();
    }
  }
}
