﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.Potvin {
	[Item("PotvinCreator", "Creator class used to create Potvin-Encoding solutions for bin packing problems.")]
	[StorableClass]
	public class PotvinRandomCreator : PackingSolutionCreator, IStochasticOperator {

		public ILookupParameter<IRandom> RandomParameter {
			get { return (LookupParameter<IRandom>)Parameters["Random"]; }
		}
		public IValueLookupParameter<IntValue> PackingItemsParameter {
			get { return (IValueLookupParameter<IntValue>)Parameters["PackingItems"]; }
		}

		[StorableConstructor]
		protected PotvinRandomCreator(bool deserializing) : base(deserializing) { }
		protected PotvinRandomCreator(PotvinRandomCreator original, Cloner cloner) : base(original, cloner) {
		}
		public override IDeepCloneable Clone(Cloner cloner) {
			return new PotvinRandomCreator(this, cloner);
		}

    public PotvinRandomCreator()
			: base() {
			Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
			Parameters.Add(new ValueLookupParameter<IntValue>("PackingItems", "The number of packing-items handled in this problem instance."));

			EncodedSolutionParameter.ActualName = "Potvin";
		}


    public static PotvinEncoding Apply(int items, IRandom random) {
      var solution = new PotvinEncoding();
		  solution.IntVector = new IntegerVector(new Permutation(PermutationTypes.Absolute, items, random));
			return solution;
		}

		protected override IPackingSolutionEncoding CreateSolution() {
			return Apply(PackingItemsParameter.ActualValue.Value, RandomParameter.ActualValue);
		}
	}
}
